(*
This small unit DotmatrX.pas intended for direct printing
from Win applications to external output devices
(usually dot-matrix printers like Epson LX-300 or Epson LX-1050+),
connected to PC through LPT-port in TEXT mode.
Also it designed to cut up wide texts into small-format paper.

Program is FREEWARE, you are FREE to use this text until sign of my
authority, which is mandatory, is present in source text.

===============================================================
Send any comments and suggestions to tersoft@yahoo.com
WWW: tersoft.virtualawe.net

(c) 1998-2000 by Igor Homyn.

===============================================================
Technical description:

Created in Delphi4. It might work in other versions, although not tested.

To use this amazing thing:
1. Add her to the project by pressing Shift+F11 or selecting "Project/Add to project...";
2. Add her name to uses clause;
3. Start preparing document with call to BeginPrint procedure;
4. Print some plain text by calling to Print (no carrige return) or
   Println (with carrige return) procedures;
5. Stop preparing document and see what you do by calling EndPrint
   procedure. For real printing press the upper-right button.
You might use other formatting routines:
LBrush - to align string to left with fixed width;
RBrush - to align string to right with fixed width;
CBrush - to align string to center with fixed width;
Fill - to form string of given width filled with specified character.
===============================================================
*)

unit dotmatrx;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls, Buttons, Placemnt;

type
  TLX300 = class(TForm)
    Panel1: TPanel;
    Panel2: TPanel;
    Edit: TRichEdit;
    Panel3: TPanel;
    BitBtn1: TBitBtn;
    SpeedButton1: TSpeedButton;
    SpeedButton2: TSpeedButton;
    OpenDialog: TOpenDialog;
    SaveDialog: TSaveDialog;
    SpeedButton3: TSpeedButton;
    PprHeight: TEdit;
    PprWidth: TEdit;
    Label1: TLabel;
    Label2: TLabel;
    Bevel1: TBevel;
    FormStorage1: TFormStorage;
    procedure FormShow(Sender: TObject);
    procedure SpeedButton1Click(Sender: TObject);
    procedure SpeedButton2Click(Sender: TObject);
    procedure SpeedButton3Click(Sender: TObject);
    procedure PprHeightChange(Sender: TObject);
    procedure BitBtn1Click(Sender: TObject);
  private
    procedure WMGetMinMaxInfo(var M: TWMGetMinMaxInfo);
                              message WM_GETMINMAXINFO;
    { Private declarations }
  public
    { Public declarations }
  end;

EEpsonNotActive = class(Exception);

procedure BeginPrn;
procedure PrnLn(S:string);
procedure Prn(S:string);
procedure EndPrn;
procedure SendToEpson(const L:TStrings);

function Brush(Kind:byte;const S:string; n:byte):string;
function LBrush(const S:string; n:byte):string;
function RBrush(const S:string; n:byte):string;
function CBrush(const S:string; n:byte):string;
function Fill(const c:char; n:integer):string;

const
  LeftMargin:byte = 0;
  PaperHeight:integer = 0;
  PaperWidth:integer = 0;
  LptNo:word = 0;
  PrnStatus:byte = 0;
  Buf:string = '';

var
  LX300: TLX300;

implementation

{$R *.DFM}

function Brush(Kind:byte;const S:string; n:byte):string;
var
  i,k:byte;
begin
  Result:=S;
  Result:=Trim(Result);
  Result:=Copy(Result,1,n);
  case Kind of
    0:while Length(Result) < n do
        Result:=' '+Result;
    1:while Length(Result) < n do
        Result:=Result+' ';
    2:begin
      k:=n-Length(Result);
      if k <= 0
        then Exit;
      for i:=1 to k div 2 do
        Result:=' '+Result;
      for i:=1 to (k div 2) + (k mod 2) do
        Result:=Result+' ';
      end;
    end;
end;

function LBrush(const S:string; n:byte):string;
begin
  Result:=Brush(1,S,n);
end;

function RBrush(const S:string; n:byte):string;
begin
  Result:=Brush(0,S,n);
end;

function CBrush(const S:string; n:byte):string;
begin
  Result:=Brush(2,S,n);
end;

function Fill(const c:char; n:integer):string;
begin
  Result:='';
  while Length(Result) < n do
    Result:=Result+c
end;

procedure BeginPrn;
begin
  LX300.Edit.Lines.Clear;
end;

procedure Prn(S:string);
begin
  Buf:=Buf+S;
end;

procedure PrnLn(S:string);
begin
  LX300.Edit.Lines.Add(Buf+S);
  Buf:='';
end;

procedure EndPrn;
begin
  LX300.ShowModal;
end;

procedure TLX300.WMGetMinMaxInfo(var M: TWMGetMinMaxInfo);
begin
  inherited;
  M.MinMaxInfo^.ptMinTrackSize.x:=435;
  M.MinMaxInfo^.ptMinTrackSize.y:=337;
end;

procedure TLX300.FormShow(Sender: TObject);
begin
  BitBtn1.SetFocus;      
end;

procedure TLX300.SpeedButton1Click(Sender: TObject);
begin
  if SaveDialog.Execute then
  begin
    if FileExists(SaveDialog.FileName) then
      if MessageDlg(Format('OK to overwrite %s', [SaveDialog.FileName]),
        mtConfirmation, mbYesNoCancel, 0) <> idYes then Exit;
    Edit.Lines.SaveToFile(SaveDialog.FileName);
    Edit.Modified:=false;
  end;
end;

procedure TLX300.SpeedButton2Click(Sender: TObject);
begin
  if OpenDialog.Execute then
  begin
    Edit.Lines.LoadFromFile(OpenDialog.FileName);
    BitBtn1.SetFocus;
    Edit.Modified := False;
    Edit.ReadOnly := ofReadOnly in OpenDialog.Options;
  end;
end;

procedure TLX300.SpeedButton3Click(Sender: TObject);
begin
  Edit.Lines.Clear;
end;

function GetPrnStatus:byte;assembler;
asm
  mov dx,LptNo
  mov ah,02
  int 17h
  xchg ah,al
end;

function PrnOut(C:char):byte;assembler;
asm
  mov dx,LptNo
  mov al,C
  mov ah,0
  int 17h
  xchg ah,al
end;

function PrnOk:boolean;
begin
 PrnStatus:=GetPrnStatus;
 Result:=true;
 if (PrnStatus <> $90)
   then Result:=false
end;

function ErrorMessage(i:byte):string;
begin
 case i of
   0:ErrorMessage:='    i';
   1:ErrorMessage:='i ';
   2:ErrorMessage:='  ';
   3:ErrorMessage:='i ';
   4:ErrorMessage:=' i ';
   5:ErrorMessage:='i   ';
   6:ErrorMessage:=' i';
   7:ErrorMessage:='i i;   ';
   8:ErrorMessage:='  ';
   9:ErrorMessage:=' i ii i';
  10:ErrorMessage:=' ';
  11:ErrorMessage:=' ';
  12:ErrorMessage:=', i ';
  else ErrorMessage:='i ';
  end;
end;

function Retry(ErrT,ErrC:word):boolean;
var
 OutStr:string[70];
begin
 if ($8000 and ErrT) = 0
   then begin
        OutStr:='   '+Chr($41+Lo(ErrT))+': * '+ErrorMessage(Lo(ErrC));
        end
   else if PrnStatus = $90
          then OutStr:='  :   FAT '
          else begin
               OutStr:=IntToHex(PrnStatus,2)+'h.  ';
               if (PrnStatus and $80) <> 1
                 then OutStr:=OutStr+'  ';
               if (PrnStatus and $40) <> 0
                 then OutStr:=OutStr+'acknwl. ';
               if (PrnStatus and $20) <> 0
                 then OutStr:=OutStr+'   ';
               if (PrnStatus and $10) <> 0
                 then OutStr:=OutStr+' ';
               if (PrnStatus and $08) <> 0
                 then OutStr:=OutStr+'I/O ';
               if (PrnStatus and $01) <> 0
                 then OutStr:=OutStr+'- ';
               if ErrC <> $8000
                 then OutStr:=OutStr+'*';
               end;
 OutStr:=TrimRight(OutStr);
 OutStr:=OutStr+'. ?';
 Result:=MessageDlg(OutStr,mtWarning, [mbRetry, mbCancel], 0) = mrRetry;
end;

function PrnSend(const s:string):boolean;
var
  k:byte;
{  prntd:boolean;}
  StrtTime:TDateTime;
  Hour, Min, Sec, MSec:word;
begin
 Result:=false;
 for k:=1 to Length(s) do
   begin
   StrtTime:=Time;
   while not PrnOk do
     begin  
     if not((PrnStatus <> $10) and (PrnStatus <> $D0) and
            (PrnStatus <> $50) and (PrnStatus <> $30) and
            (PrnStatus <> $70))
       then begin
            DecodeTime(Time-StrtTime, Hour, Min, Sec, MSec);
            if Sec > 5
              then begin
                   if not Retry($8000,$7000)
                     then Exit
                     else StrtTime:=Time;
                   end
            end
       else begin
            if not Retry($8000,$8000)
              then Exit;
            end
     end;
   PrnStatus:=PrnOut(s[k]);
   end;
 Result:=true;
end;

function Uncode(const s:string):string;
var
  i:byte;
  c:char;
begin
  Result:='';
  for i:=1 to Length(s) do
    begin
    case s[i] of
  '':c:=Chr($80); '':c:=Chr($81); '':c:=Chr($82); '':c:=Chr($83);
  '':c:=Chr($84); '':c:=Chr($85); '':c:=Chr($86); '':c:=Chr($87);
  '':c:=Chr($88); '':c:=Chr($89); '':c:=Chr($8A); '':c:=Chr($8B);
  '':c:=Chr($8C); '':c:=Chr($8D); '':c:=Chr($8E); '':c:=Chr($8F);

  '':c:=Chr($90); '':c:=Chr($91); '':c:=Chr($92); '':c:=Chr($93);
  '':c:=Chr($94); '':c:=Chr($95); '':c:=Chr($96); '':c:=Chr($97);
  '':c:=Chr($98); '':c:=Chr($99); '':c:=Chr($9A); '':c:=Chr($9B);
  '':c:=Chr($9C); '':c:=Chr($9D); '':c:=Chr($9E); '':c:=Chr($9F);

  '':c:=Chr($A0); '':c:=Chr($A1); '':c:=Chr($A2); '':c:=Chr($A3);
  '':c:=Chr($A4); '':c:=Chr($A5); '':c:=Chr($A6); '':c:=Chr($A7);
  '':c:=Chr($A8); '':c:=Chr($A9); '':c:=Chr($AA); '':c:=Chr($AB);
  '':c:=Chr($AC); '':c:=Chr($AD); '':c:=Chr($AE); '':c:=Chr($AF);

  '':c:=Chr($E0); '':c:=Chr($E1); '':c:=Chr($E2); '':c:=Chr($E3);
  '':c:=Chr($E4); '':c:=Chr($E5); '':c:=Chr($E6); '':c:=Chr($E7);
  '':c:=Chr($E8); '':c:=Chr($E9); '':c:=Chr($EA); '':c:=Chr($EB);
  '':c:=Chr($EC); '':c:=Chr($ED); '':c:=Chr($EE); '':c:=Chr($EF);

  '':c:=Chr($F0); '':c:=Chr($F1); '':c:=Chr($F2); '':c:=Chr($F3);
  '':c:=Chr($F4); '':c:=Chr($F5);
  '':c:=Chr($FC); '':c:=Chr($49); '':c:=Chr($69);
      else c:=s[i];
      end;
    Result:=Result+c;
    end;
end;

{ $define TEST}
{$ifdef TEST}
var
  PrnF:text;
{$endif}

procedure HardBeginPrint;
begin
{$ifdef TEST}
AssignFile(PrnF,'TEST.TXT');
Append(PrnF);
{$endif}
end;

function HardPrint(const s:string):boolean;
var
 s1:string;
begin
s1:=Uncode(s);
{$ifdef TEST}
Writeln(PrnF,s1);
Result:=true;
{$else}
 Result:=false;
 if not PrnSend(Fill(' ',LeftMargin)+s1) then Exit;
 if not PrnSend(Chr(13)) then Exit;
 if not PrnSend(Chr(10)) then Exit;
 Result:=true;
{$endif}
end;

procedure HardEndPrint;
begin
{$ifdef TEST}
Writeln(PrnF,'*-*-*-*-*-*-**--');
Close(PrnF);
{$endif}
end;

function GetLine(const L:TStrings; const i,Start,Len:word):string;
begin
  if Len = 0
    then Result:=L[i]
    else Result:=Copy(L[i],Start,Len);
end;

function GetMaxWidth(const L:TStrings;const ThisPgStart,ThisPgEnd:integer):integer;
var
  i:integer;
begin
  Result:=Length(L[ThisPgStart]);
  for i:=ThisPgStart+1 to ThisPgEnd do
    if Result < Length(L[i])
      then Result:=Length(L[i]);
end;

procedure SendToEpson(const L:TStrings);
var
  ThisPgStart,ThisPgEnd,
  CurLine,PgNo,LocalPgNo,
  MaxWidth,PagesWide:integer;
begin
  ThisPgStart:=1;
  if L.Count < PaperHeight
    then ThisPgEnd:=L.Count
    else ThisPgEnd:=PaperHeight;
  if ThisPgEnd = 0
    then ThisPgEnd:=L.Count;
  PgNo:=1;{LocalPgNo:=1;}
  while ThisPgStart <= L.Count do
    begin
    if PaperWidth <> 0
      then begin
           MaxWidth:=GetMaxWidth(L,ThisPgStart,ThisPgEnd);
           PagesWide:=MaxWidth div PaperWidth;
           if (MaxWidth mod PaperWidth) > 0
             then PagesWide:=PagesWide+1;
           end
      else PagesWide:=1;
    for LocalPgNo:=1 to PagesWide do
      begin
      if PagesWide = 1
        then begin
             if (PgNo <> 1)
                then begin
                     if MessageDlg('  '+IntToStr(PgNo)+'. ?',
                       mtConfirmation, [mbOk, mbCancel], 0) = mrCancel
                       then Exit;
                     end
             end
        else begin
             if MessageDlg('  '+IntToStr(PgNo)+'  '+IntToStr(LocalPgNo)+'. ?',
                       mtConfirmation, [mbOk, mbCancel], 0) = mrCancel
               then Exit;
             end;
      HardBeginPrint;
      for CurLine:=ThisPgStart to ThisPgEnd do
        begin
        HardPrint(GetLine(L,CurLine-1,(LocalPgNo-1)*PaperWidth+1,PaperWidth));
        end;
      HardEndPrint;
      end;

    ThisPgStart:=ThisPgEnd+1;
    if ThisPgStart-1+PaperHeight > L.Count
      then ThisPgEnd:=L.Count
      else ThisPgEnd:=ThisPgStart-1+PaperHeight;
    inc(PgNo);
    end;
end;

procedure TLX300.PprHeightChange(Sender: TObject);
begin
  PaperHeight:=StrToIntDef(PprHeight.Text,0);
  PaperWidth:=StrToIntDef(PprWidth.Text,0);
  if PaperHeight < 0 then PaperHeight:=0;
  if PaperWidth < 0 then PaperWidth:=0;
end;

procedure TLX300.BitBtn1Click(Sender: TObject);
begin
  SendToEpson(Edit.Lines);
end;

end.
